@@ -38,6 +38,7 @@ class Puppet::Application::Agent < Puppet::Application
       :digest => 'SHA256',
       :graph => true,
       :fingerprint => false,
+      :sourceaddress => nil,
     }.each do |opt,val|
       options[opt] = val
     end
@@ -59,6 +60,8 @@ class Puppet::Application::Agent < Puppet::Application
   option("--fingerprint")
   option("--digest DIGEST")
 
+  option("--sourceaddress IP_ADDRESS")
+
   option("--detailed-exitcodes") do |arg|
     options[:detailed_exitcodes] = true
   end
@@ -98,8 +101,8 @@ USAGE
 -----
 puppet agent [--certname <NAME>] [-D|--daemonize|--no-daemonize]
   [-d|--debug] [--detailed-exitcodes] [--digest <DIGEST>] [--disable [MESSAGE]] [--enable]
-  [--fingerprint] [-h|--help] [-l|--logdest syslog|eventlog|<FILE>|console]
-  [--masterport <PORT>] [--noop] [-o|--onetime] [-t|--test]
+  [--fingerprint] [-h|--help] [-l|--logdest syslog|eventlog|<ABS FILEPATH>|console]
+  [--masterport <PORT>] [--noop] [-o|--onetime] [--sourceaddress <IP_ADDRESS>] [-t|--test]
   [-v|--verbose] [-V|--version] [-w|--waitforcert <SECONDS>]
 
 
@@ -163,7 +166,7 @@ specify '--server <servername>' as an argument. Boolean settings translate into
 '--setting' and '--no-setting' pairs.
 
 See the configuration file documentation at
-https://docs.puppetlabs.com/puppet/latest/reference/configuration.html for the
+https://puppet.com/docs/puppet/latest/configuration.html for the
 full list of acceptable settings. A commented list of all settings can also be
 generated by running puppet agent with '--genconfig'.
 
@@ -273,6 +276,10 @@ generated by running puppet agent with '--genconfig'.
   (This is a Puppet setting, and can go in puppet.conf. Note the special 'no-'
   prefix for boolean settings on the command line.)
 
+* --sourceaddress:
+  Set the source IP address for transactions. This defaults to automatically selected.
+  (This is a Puppet setting, and can go in puppet.conf.)
+
 * --test:
   Enable the most common options used for testing. These are 'onetime',
   'verbose', 'no-daemonize', 'no-usecacheonfailure', 'detailed-exitcodes',
@@ -415,18 +422,12 @@ Copyright (c) 2011 Puppet Inc., LLC Licensed under the Apache 2.0 License
     # we want the last report to be persisted locally
     Puppet::Transaction::Report.indirection.cache_class = :yaml
 
-    if Puppet[:noop]
-      Puppet::Resource::Catalog.indirection.cache_class = nil
-    elsif Puppet[:catalog_cache_terminus]
+    if Puppet[:catalog_cache_terminus]
       Puppet::Resource::Catalog.indirection.cache_class = Puppet[:catalog_cache_terminus]
     end
 
-    if options[:fingerprint]
-      # in fingerprint mode we just need
-      # access to the local files and we don't need a ca
-      Puppet::SSL::Host.ca_location = :none
-    else
-      Puppet::SSL::Host.ca_location = :remote
+    # In fingerprint mode we don't need to set up the whole agent
+    unless options[:fingerprint]
       setup_agent
     end
   end
