@@ -3,7 +3,7 @@ module Puppet
   class Error < RuntimeError
     attr_accessor :original
     def initialize(message, original=nil)
-      super(Puppet::Util::CharacterEncoding.scrub(message))
+      super(message.scrub)
       @original = original
     end
   end
@@ -12,7 +12,7 @@ module Puppet
     # This module implements logging with a filename and line number. Use this
     # for errors that need to report a location in a non-ruby file that we
     # parse.
-    attr_accessor :line, :file, :pos, :puppetstack
+    attr_accessor :line, :file, :pos
 
     # May be called with 3 arguments for message, file, line, and exception, or
     # 4 args including the position on the line.
@@ -22,18 +22,10 @@ module Puppet
         original = pos
         pos = nil
       end
-
       super(message, original)
-
       @file = file unless (file.is_a?(String) && file.empty?)
       @line = line
       @pos = pos
-
-      if original && original.respond_to?(:puppetstack)
-        @puppetstack = original.puppetstack
-      else
-        @puppetstack = Puppet::Pops::PuppetStack.stacktrace()
-      end
     end
 
     def to_s
@@ -110,4 +102,16 @@ module Puppet
   class LockError < Puppet::Error
   end
 
+  # An Error suitable for raising an error with details in a Puppet::Datatypes::Error
+  # that can be used as a value in the Puppet Language
+  #
+  class ErrorWithData < Puppet::Error
+    include ExternalFileError
+    attr_reader :error_data
+
+    def initialize(error_data, message, file: nil, line: nil, pos: nil, original:nil)
+      super(message, file, line, pos, original)
+      @error_data = error_data
+    end
+  end
 end
