@@ -6,7 +6,9 @@ module Puppet::Interface::FaceCollection
   def self.faces
     unless @loaded
       @loaded = true
-      names = @loader.files_to_load.map {|fn| ::File.basename(fn, '.rb')}.uniq
+      names = @loader.files_to_load(Puppet.lookup(:current_environment)).map do |fn|
+        ::File.basename(fn, '.rb')
+      end.uniq
       names.each {|name| self[name, :current]}
     end
     @faces.keys.select {|name| @faces[name].length > 0 }
@@ -111,7 +113,7 @@ module Puppet::Interface::FaceCollection
     nil
   rescue SyntaxError => e
     raise unless e.message =~ %r{#{path}\.rb:\d+: }
-    Puppet.err "Failed to load face #{name}:\n#{e}"
+    Puppet.err _("Failed to load face %{name}:\n%{detail}") % { name: name, detail: e }
     # ...but we just carry on after complaining.
     nil
   end
@@ -122,7 +124,9 @@ module Puppet::Interface::FaceCollection
 
   def self.underscorize(name)
     unless name.to_s =~ /^[-_a-z][-_a-z0-9]*$/i then
-      raise ArgumentError, "#{name.inspect} (#{name.class}) is not a valid face name"
+      #TRANSLATORS 'face' refers to a programming API in Puppet
+      raise ArgumentError, _("%{name} (%{class_name}) is not a valid face name") %
+          { name: name.inspect, class_name: name.class }
     end
 
     name.to_s.downcase.split(/[-_]/).join('_').to_sym
