@@ -64,6 +64,6 @@ class Puppet::Pops::Evaluator::Collectors::ExportedCollector < Puppet::Pops::Eva
   end
 
   def to_s
-    "Exported-Collector[#{@type}]"
+    "Exported-Collector[#{@type.to_s}]"
   end
 end
