@@ -14,7 +14,6 @@ Puppet::Type.type(:package).provide :rpm, :source => :rpm, :parent => Puppet::Pr
   has_feature :install_options
   has_feature :uninstall_options
   has_feature :virtual_packages
-  has_feature :install_only
 
   # Note: self:: is required here to keep these constants in the context of what will
   # eventually become this Puppet::Type::Package::ProviderRpm class.
@@ -22,7 +21,6 @@ Puppet::Type.type(:package).provide :rpm, :source => :rpm, :parent => Puppet::Pr
   self::NEVRA_FORMAT = %Q{%{NAME} %|EPOCH?{%{EPOCH}}:{0}| %{VERSION} %{RELEASE} %{ARCH}\\n}
   self::NEVRA_REGEX  = %r{^'?(\S+) (\S+) (\S+) (\S+) (\S+)$}
   self::NEVRA_FIELDS = [:name, :epoch, :version, :release, :arch]
-  self::MULTIVERSION_SEPARATOR = "; "
 
   ARCH_LIST = [
     'noarch',
@@ -82,9 +80,12 @@ Puppet::Type.type(:package).provide :rpm, :source => :rpm, :parent => Puppet::Pr
 
     # list out all of the packages
     begin
-      execpipe("#{command(:rpm)} -qa #{nosignature} #{nodigest} --qf '#{self::NEVRA_FORMAT}' | sort") { |process|
+      execpipe("#{command(:rpm)} -qa #{nosignature} #{nodigest} --qf '#{self::NEVRA_FORMAT}'") { |process|
         # now turn each returned line into a package object
-        nevra_to_multiversion_hash(process).each { |hash| packages << new(hash) }
+        process.each_line { |line|
+          hash = nevra_to_hash(line)
+          packages << new(hash) unless hash.empty?
+        }
       }
     rescue Puppet::ExecutionFailure
       raise Puppet::Error, _("Failed to list packages"), $!.backtrace
@@ -100,7 +101,7 @@ Puppet::Type.type(:package).provide :rpm, :source => :rpm, :parent => Puppet::Pr
     #NOTE: Prior to a fix for issue 1243, this method potentially returned a cached value
     #IF YOU CALL THIS METHOD, IT WILL CALL RPM
     #Use get(:property) to check if cached values are available
-    cmd = ["-q",  @resource[:name], "#{self.class.nosignature}", "#{self.class.nodigest}", "--qf", "#{self.class::NEVRA_FORMAT}"]
+    cmd = ["-q",  @resource[:name], "#{self.class.nosignature}", "#{self.class.nodigest}", "--qf", "'#{self.class::NEVRA_FORMAT}'"]
 
     begin
       output = rpm(*cmd)
@@ -117,7 +118,9 @@ Puppet::Type.type(:package).provide :rpm, :source => :rpm, :parent => Puppet::Pr
         return nil
       end
     end
-    @property_hash.update(self.class.nevra_to_multiversion_hash(output))
+    # FIXME: We could actually be getting back multiple packages
+    # for multilib and this will only return the first such package
+    @property_hash.update(self.class.nevra_to_hash(output))
 
     @property_hash.dup
   end
@@ -128,8 +131,8 @@ Puppet::Type.type(:package).provide :rpm, :source => :rpm, :parent => Puppet::Pr
       @resource.fail _("RPMs must specify a package source")
     end
 
-    cmd = [command(:rpm), "-q", "--qf", "#{self.class::NEVRA_FORMAT}", "-p", source]
-    h = self.class.nevra_to_multiversion_hash(execute(cmd))
+    cmd = [command(:rpm), "-q", "--qf", "'#{self.class::NEVRA_FORMAT}'", "-p", source]
+    h = self.class.nevra_to_hash(execute(cmd))
     h[:ensure]
   rescue Puppet::ExecutionFailure => e
     raise Puppet::Error, e.message, e.backtrace
@@ -152,43 +155,28 @@ Puppet::Type.type(:package).provide :rpm, :source => :rpm, :parent => Puppet::Pr
   end
 
   def uninstall
-    query
-    # If version and release (or only version) is specified in the resource,
-    # uninstall using them, otherwise uninstall using only the name of the package.
-    name    = get(:name)
-    version = get(:version)
-    release = get(:release)
-    nav = "#{name}-#{version}"
-    nvr = "#{nav}-#{release}"
-    if @resource[:name].start_with? nvr
-      identifier = nvr
-    else
-      if @resource[:name].start_with? nav
-        identifier = nav
-      else
-        if @resource[:install_only]
-          identifier = get(:ensure).split(self.class::MULTIVERSION_SEPARATOR).map { |ver| "#{name}-#{ver}" }
-        else
-          identifier = name
-        end
-      end
-    end
-    # If an arch is specified in the resource, uninstall that arch,
-    # otherwise uninstall the arch returned by query.
-    # If multiple arches are installed and arch is not specified,
-    # this will uninstall all of them after successive runs.
-    #
-    # rpm prior to 4.2.1 cannot accept architecture as part of the package name.
+    query if get(:arch) == :absent
+    nvr = "#{get(:name)}-#{get(:version)}-#{get(:release)}"
+    arch = ".#{get(:arch)}"
+    # If they specified an arch in the manifest, erase that Otherwise,
+    # erase the arch we got back from the query. If multiple arches are
+    # installed and only the package name is specified (without the
+    # arch), this will uninstall all of them on successive runs of the
+    # client, one after the other
+
+    # version of RPM prior to 4.2.1 can't accept the architecture as
+    # part of the package name.
     unless Puppet::Util::Package.versioncmp(self.class.current_version, '4.2.1') < 0
-      arch = ".#{get(:arch)}"
-      if @resource[:name].end_with? arch
-        identifier += arch
+      if @resource[:name][-arch.size, arch.size] == arch
+        nvr += arch
+      else
+        nvr += ".#{get(:arch)}"
       end
     end
 
     flag = ['-e']
     flag += uninstall_options if resource[:uninstall_options]
-    rpm flag, identifier
+    rpm flag, nvr
   end
 
   def update
@@ -229,14 +217,14 @@ Puppet::Type.type(:package).provide :rpm, :source => :rpm, :parent => Puppet::Pr
       str2 = str2.gsub(front_strip_re, '')
 
       # "handle the tilde separator, it sorts before everything else"
-      if str1 =~ /^~/ && str2 =~ /^~/
+      if /^~/.match(str1) && /^~/.match(str2)
         # if they both have ~, strip it
         str1 = str1[1..-1]
         str2 = str2[1..-1]
         next
-      elsif str1 =~ /^~/
+      elsif /^~/.match(str1)
         return -1
-      elsif str2 =~ /^~/
+      elsif /^~/.match(str2)
         return 1
       end
 
@@ -245,7 +233,7 @@ Puppet::Type.type(:package).provide :rpm, :source => :rpm, :parent => Puppet::Pr
       # "grab first completely alpha or completely numeric segment"
       isnum = false
       # if the first char of str1 is a digit, grab the chunk of continuous digits from each string
-      if str1 =~ /^[0-9]+/
+      if /^[0-9]+/.match(str1)
         if str1 =~ /^[0-9]+/
           segment1 = $~.to_s
           str1 = $~.post_match
@@ -310,12 +298,8 @@ Puppet::Type.type(:package).provide :rpm, :source => :rpm, :parent => Puppet::Pr
 
   def insync?(is)
     return false if [:purged, :absent].include?(is)
-    return false if is.include?(self.class::MULTIVERSION_SEPARATOR) && !@resource[:install_only]
-
     should = resource[:ensure]
-    is.split(self.class::MULTIVERSION_SEPARATOR).any? do |version|
-      0 == self.rpm_compareEVR(rpm_parse_evr(should), rpm_parse_evr(version))
-    end
+    0 == rpm_compareEVR(rpm_parse_evr(should), rpm_parse_evr(is))
   end
 
   # parse a rpm "version" specification
@@ -340,7 +324,7 @@ Puppet::Type.type(:package).provide :rpm, :source => :rpm, :parent => Puppet::Pr
       v = s[0,ri]
       r = s[ri+1,s.length]
       if arch = r.scan(ARCH_REGEX)[0]
-        a = arch.delete('.')
+        a = arch.gsub(/\./, '')
         r.gsub!(ARCH_REGEX, '')
       end
     else
@@ -418,37 +402,4 @@ Puppet::Type.type(:package).provide :rpm, :source => :rpm, :parent => Puppet::Pr
 
     return hash
   end
-
-  # @param line [String] multiple lines of rpm package query information
-  # @return list of [Hash] of NEVRA_FIELDS strings parsed from package info
-  # or an empty list if we failed to parse
-  # @api private
-  def self.nevra_to_multiversion_hash(multiline)
-    list = []
-    multiversion_hash = {}
-    multiline.each_line do |line|
-      hash = self.nevra_to_hash(line)
-      if !hash.empty?
-        if multiversion_hash.empty?
-          multiversion_hash = hash.dup
-          next
-        end
-
-        if multiversion_hash[:name] != hash[:name]
-          list << multiversion_hash
-          multiversion_hash = hash.dup
-          next
-        end
-
-        if !multiversion_hash[:ensure].include?(hash[:ensure])
-          multiversion_hash[:ensure].concat("#{self::MULTIVERSION_SEPARATOR}#{hash[:ensure]}")
-        end
-      end
-    end
-    list << multiversion_hash if multiversion_hash
-    if list.size == 1
-      return list[0]
-    end
-    return list
-  end
 end
