@@ -144,11 +144,6 @@ module Puppet::Test
       Puppet::Util::Profiler.clear
 
       Puppet::SSL::Host.reset
-      Puppet::SSL::Host.ca_location = :none
-
-      Puppet::Node::Facts.indirection.terminus_class = :memory
-      facts = Puppet::Node::Facts.new(Puppet[:node_name_value])
-      Puppet::Node::Facts.indirection.save(facts)
 
       Puppet.clear_deprecation_warnings
     end
