@@ -1,3 +1,4 @@
+# coding: utf-8
 require 'digest/md5'
 require 'cgi'
 require 'etc'
@@ -12,7 +13,6 @@ require 'puppet/util/backups'
 require 'puppet/util/symbolic_file_mode'
 
 Puppet::Type.newtype(:file) do
-  include Puppet::Util::MethodHelper
   include Puppet::Util::Checksums
   include Puppet::Util::Backups
   include Puppet::Util::SymbolicFileMode
@@ -572,7 +572,8 @@ Puppet::Type.newtype(:file) do
     # remote system.
     mark_children_for_purging(children) if self.purge?
 
-    result = children.values.sort_by { |a| a[:path] }
+    # REVISIT: sort_by is more efficient?
+    result = children.values.sort { |a, b| a[:path] <=> b[:path] }
     remove_less_specific_files(result)
   end
 
@@ -748,7 +749,7 @@ Puppet::Type.newtype(:file) do
     # This check is done in retrieve to ensure it happens before we try to use
     # metadata in `copy_source_values`, but so it only fails the resource and not
     # catalog validation (because that would be a breaking change from Puppet 4).
-    if Puppet.features.microsoft_windows? && parameter(:source) &&
+    if Puppet::Util::Platform.windows? && parameter(:source) &&
       [:use, :use_when_creating].include?(self[:source_permissions])
       #TRANSLATORS "source_permissions => ignore" should not be translated
       err_msg = _("Copying owner/mode/group from the source file on Windows is not supported; use source_permissions => ignore.")
@@ -834,6 +835,9 @@ Puppet::Type.newtype(:file) do
     rescue Errno::EACCES
       warning _("Could not stat; permission denied")
       nil
+    rescue Errno::EINVAL
+      warning _("Could not stat; invalid pathname")
+      nil
     end
   end
 
